# -*- coding: utf-8 -*-
# Copyright (C) 2005 Lateef Alabi-Oki
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Main entry point to program

This module contains code that generates new instances of the editor. It also
details basic command line operations that the editor can perform. When Scribes
is executed, main() is called. If Scribes is executed without command line
arguments, main() calls create_new_editor(). Otherwise, main() processes the
command line options, if any, and then calls launch_new_file().

Command line options like --help or --version only output text to the console.
They do not create a new instance of the editor.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sys import exit
from os import path, fork
from locale import getpreferredencoding
from gnomevfs import get_uri_from_local_path

from pygtk import require
require("2.0")

from info import version
from internationalization import msg0038, msg0039, msg0040, msg0041, msg0042
from internationalization import msg0043
from usage import help as chelp
from fileloader import FileLoader
from editor import Editor


def create_new_editor():
	"""
	Create a new instance of the editor

	@return: a new editor window
	@rtype: editor object
	"""

	editor = Editor()

	editor.show_all()
	editor.window.process_updates(True)

	return	editor


def launch_new_file(file_uri, readonly):
	"""
	Create a new instance of the editor with a file opened

	@param filename: name of a text file to open
	@type filename: string

	@return: a new editor window with an open text file
	@rtype: editor object
	"""

	editor = Editor()
	editor.readonly = readonly
	FileLoader(editor).check_load_flag(file_uri)
	editor.show_all()
	editor.window.process_updates(True)

	return editor


def create_new_file(filename):
	"""
	Create a new file

	@param filename: name of the file to create
	@type filename: string
	"""

	newfile = open(filename, "w")
	newfile.close()

	return


def process_command_line_options(argv):
	"""
	Analyze command line options and process them.

	@param argv: command line arguments
	@type argv: list

	@return: None, or a flag if "-n" or "-r" exists in argv.
	@rtype: None, or boolean
	"""

	# Command line options.
	options = ["-n", "--newfile",
			"-r", "--readonly",
			"-v", "--version",
			"-h", "--help"]

	# Number of options found.
	count = 0
	flag = None
	encoding = getpreferredencoding(True)

	for filename in argv:

		# Exit the program with an error message if unrecognized options are
		# passed as arguments.
		if filename.startswith("-") and filename not in options:

			# See internationalization.py for the error messages.
			print msg0038.decode("utf-8").encode(encoding) % filename
			print msg0039.decode("utf-8").encode(encoding)
			exit(1)

		# Ensure we accept only one option at a time.
		if filename.startswith("-"):

			count += 1

			if count > 1:

				# See internationalization.py for the error messages.
				print msg0040.decode("utf-8").encode(encoding)
				print msg0039.decode("utf-8").encode(encoding)
				exit(1)

		# Print the command line help file to the shell console.
		if filename == "-h" or filename == "--help":

			if len(argv) > 1:

				# See internationalization.py for the error messages.
				print msg0041.decode("utf-8").encode(encoding) % filename
				print msg0039.decode("utf-8").encode(encoding)
				exit(1)

			chelp()
			exit(1)

		# Print the version of Scribes installed on the host system.
		if filename == "-v" or filename == "--version":

			if len(argv) > 1:

				# See internationalization.py for the error messages.
				print msg0041.decode("utf-8").encode(encoding) % filename
				print msg0039.decode("utf-8").encode(encoding)
				exit(1)

			print msg0042.decode("utf-8").encode(encoding) % \
version.encode(encoding, "replace")
			exit(1)

		# Enable the editor create new files.
		if filename == "-n" or filename == "--newfile":

			flag = True

		# Enable the editor launch files in readonly mode.
		if filename == "-r" or filename == "--readonly":

			flag = True

	return flag


def fork_scribes():
	"""
	Fork Scribes' process.

	"""

	pid = fork()

	if pid != 0:

		exit(0)

	return


def main(argv):
	"""
	Entry point to the program

	@param argv: a list of text files to be opened for viewing or editing
	@type argv:	list

	"""

	encoding = getpreferredencoding(True)

	# If the editor is called with no arguments, open the editor with an empty
	# file. If arguments are given and the arguments are filenames, then
	# open each file in a separate editor window.
	if not len(argv):

		fork_scribes()
		editor = create_new_editor()
		editor.gui_mainloop()

	if len(argv):

		# A flag set to True when the "-n" option is passed to arguments at the
		# prompt in a shell terminal.
		newfile_flag = False

		# A flag set to True when the "-r" option is passed to arguments at the
		# prompt in a shell terminal.
		readonly_flag = False

		from collections import deque

		# A container housing editor objects with open text files
		file_holder = deque([])

		# Process command line options.
		flag = process_command_line_options(argv)
		fork_scribes()

		if "-n" in argv or "--newfile" in argv:

			newfile_flag = flag

		if "-r" in argv or "--readonly" in argv:

			readonly_flag = flag

		# Prepare files for loading.
		for filename in argv:

			# If the newfile_flag is set to True, create files that do not
			# exist, but that are passed as arguments at the prompt in a shell
			# terminal.
			if newfile_flag:

				for files in argv:

					if files.startswith("-"):

						continue

					else:

						if not path.exists(files.strip()):

							create_new_file(files)

				newfile_flag = False

			# Ignore option flags passed as arguments at the prompt in a shell
			# terminal.
			if filename.startswith("-"):

				continue

			# Load the files passed as arguments at the prompt in a shell
			# terminal.
			if path.exists(filename.strip()):

				file_path = path.abspath(filename)
				file_uri = get_uri_from_local_path(file_path)
				editor = launch_new_file(file_uri, readonly_flag)
				file_holder.append(editor)

			else:

				# See internationalization.py for the error messages.
				print msg0043.decode("utf-8").encode(encoding) % filename

		# Connect editor objects to the GTK mainloop
		for editor in file_holder:

			editor.gui_mainloop()

	return

