#!/usr/bin/env python 
# -*- coding: utf-8 -*-
#
# This file is part of Scribes.
#
# Scribes is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Scribes is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Scribes; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""
Installation script for Scribes.

This module installs Scribes' data files and modules on the host system.
Software dependency checks are also performed to ensure that the host system has
all the required software to run Scribes.

Scribes depends on GNOME version 2.10, Python version 2.4, PyGTK version 2.6,
GNOME Python version 2.10 and GNOME Python Extras version 2.10 to run.

@author: Lateef Alabi-Oki
@organiation: The Scribes Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from sys import version_info, exit
from os import putenv
from commands import getstatusoutput
from distutils.core import setup
from Scribes.info import version


def check_dependencies():
	u"""
	Check for Scribes' runtime software dependencies.

	The function aborts the installation process if runtime software
	dependencies are not met by the host system.

	"""

	try:

		import pygtk
		pygtk.require("2.0")
		import gtk

		if gtk.pygtk_version < (2, 8, 0):

			raise ImportError

	except ImportError:

		print "You need PyGTK version 2.8 or later to run Scribes."
		exit(1)

	try:

		import pygtk
		pygtk.require("2.0")
		import gnome

		if gnome.gnome_python_version < (2, 12, 0):

			raise ImportError

	except ImportError:

		print "You need GNOME Python version 2.12 or later to run Scribes"
		exit(1)

	try:

		import pygtk
		pygtk.require("2.0")
		import gtksourceview

	except ImportError:

		print "You need the gtksourceview module in GNOME Python Extras version"
		print " 2.12 or later to run Scribes."
		exit(1)

	try:

		import pygtk
		pygtk.require("2.0")
		import gtkspell

	except ImportError:

		print "You need the gtkspell module in GNOME Python Extras version"
		print " 2.12 or later to run Scribes."
		exit(1)

	# Check to see if Yelp is installed on the host system

	cmd = "which yelp"
	err, out = getstatusoutput(cmd)

	if err:

		print "You need Yelp version 2.12 or later to run Scribes."
		exit(1)

	# Check to see if GConf is installed on the host system

	cmd = "which gconftool-2"
	err, out = getstatusoutput(cmd)

	if err:

		print "You need GConf version 2.12 or later to run Scribes."
		exit(1)

	return

check_dependencies()

# Trove classification

classifiers = """
Development Status :: 1 - Alpha
Environment :: X11 Applications :: GTK
Intended Audience :: Developers
Intended Audience :: Education
Intended Audience :: Information Technology
Intended Audience :: End Users/Desktop
Intended Audience :: Science/Research
Intended Audience :: System Administrators
OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Operating System :: POSIX :: Linux
Programming Language :: Python
Topic :: Documentation
Topic :: Software Development
Topic :: Text Editors

"""


################################################################################
#
#								Setup
#
################################################################################


setup(name="scribes",
	version=version,
	description="Text editor for GNOME.",
	long_description="Scribes is a general-purpose text editor for GNOME",
	classifiers = filter(None, classifiers.split('\n')),
	author="Lateef Alabi-Oki",
	author_email="mystilleef@gmail.com",
	url="http://scribes.sourceforge.net/",
	license="GNU GPL v2 and above",
	packages=["Scribes"],
	data_files=[
		("share/applications", ["data/scribes.desktop"]),
		("share/application-registry", ["data/scribes.application"]),
		("share/pixmaps", ["data/scribes.svg"]),
		("share/scribes/", ["data/scribes_toolbar_ui.xml"]),
		("share/gnome/help/scribes/C", ["data/scribes.xml"]),
		("share/gnome/help/scribes/C", ["data/legal.xml"]),
		("share/locale/ru/LC_MESSAGES/",["data/po/ru/scribes.mo"]),
		("share/locale/pt_BR/LC_MESSAGES/",["data/po/pt_BR/scribes.mo"]),
		("share/locale/de/LC_MESSAGES/",["data/po/de/scribes.mo"]),
		("share/locale/it/LC_MESSAGES/",["data/po/it/scribes.mo"]),
		("share/locale/fr/LC_MESSAGES/",["data/po/fr/scribes.mo"]),
		("share/omf/scribes", ["data/scribes-C.omf"]),
		("share/scribes/", ["data/bookmarks.png"]),
		("share/gnome/help/scribes/C/figures",
		["data/scribes_window.png", "data/scribes_toolbar.png",
		"data/scribes_editing.png", "data/scribes_status.png"])
	],
	scripts=["scribes"],)


################################################################################
#
#							GConf Installation
#
################################################################################

# Get gconf's default source

cmd = "gconftool-2 --get-default-source"
err, out = getstatusoutput(cmd)

# Set up the gconf environment variable.

putenv('GCONF_CONFIG_SOURCE', out)

# Install gconf to the default source

cmd = "gconftool-2 --makefile-install-rule data/scribes.schemas"
err, out = getstatusoutput(cmd)

if out:

	print "installing GConf schema files"

if err:

    print 'Error: installation of gconf schema files failed: %s' % out

# Kill the GConf daemon

cmd = "killall gconfd-2"
err, out = getstatusoutput(cmd)

if err:

	print "Problem shutting down gconf."

# Start the GConf daemon

cmd = "gconftool-2 --spawn"
err, out = getstatusoutput(cmd)

if err:

	print "Problem restarting down gconf."

print "scribes installation complete"

